import 'dart:async';
import 'package:flutter/material.dart';
import 'package:intl/date_symbol_data_local.dart';
import 'package:intl/intl.dart';
import 'package:flutter_picker/flutter_picker.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:factory_signage_app/const/colors.dart';
import 'package:factory_signage_app/const/config.dart';
import 'package:factory_signage_app/const/dimens.dart';
import 'package:factory_signage_app/provider.dart';

class Date extends ConsumerStatefulWidget {
  const Date(
      {super.key, this.size = const Size(double.infinity, double.infinity)});
  final Size size;

  @override
  DateState createState() => DateState();
}

class DateState extends ConsumerState<Date> {
  bool _isChangeDateTime = false;
  bool _isSelectDateTime = false;
  int _clickCount = 0;
  DateTime _setDateTime = DateTime.now();
  late int _prevMinute = 99;
  late Timer _clickCountTimer;
  late Timer _timer;

  @override
  void initState() {
    super.initState();

    _timer = Timer.periodic(
      const Duration(seconds: intervalUpdateDate),
      (timer) {
        if (!_isSelectDateTime) {
          _setDateTime = DateTime.now();
        } else {
          _setDateTime =
              _setDateTime.add(const Duration(seconds: intervalUpdateDate));
        }
        // Change provider data if different from previous minutes
        // or if time is changed by flutter picker.
        if (_setDateTime.minute != _prevMinute || _isChangeDateTime) {
          _isChangeDateTime = false;
          _prevMinute = _setDateTime.minute;
          ref.watch(presentDateTimeProvider.notifier).state = _setDateTime;
        }
      },
    );

    _clickCountTimer = Timer.periodic(
      const Duration(seconds: clickCountDuration),
      (timer) => _clickCount = 0,
    );
  }

  @override
  void dispose() {
    _timer.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final Size widgetSize = MediaQuery.of(context).size;

    return GestureDetector(
      onSecondaryTap: () {
        settingDateWithManual();
      },
      child: date(
        ref,
        widgetSize,
      ),
    );
  }

  Widget date(WidgetRef ref, Size widgetSize) {
    initializeDateFormatting("ja");
    final DateTime presentDateTime = ref.watch(presentDateTimeProvider);

    return Text(
      // ignore: prefer_interpolation_to_compose_strings
      DateFormat.yMMMd("ja").format(presentDateTime) +
          '(${DateFormat.E("ja").format(presentDateTime)})  ' +
          presentDateTime.hour.toString().padLeft(2, '0') +
          ':' +
          presentDateTime.minute.toString().padLeft(2, '0'),
      style: TextStyle(
        color: textWhiteColor,
        fontSize: titleFontSizeRatio * widgetSize.height,
      ),
    );
  }

  void settingDateWithManual() {
    _clickCountTimer.cancel();
    _clickCount++;
    _clickCountTimer = Timer.periodic(
      const Duration(seconds: clickCountDuration),
      (timer) => _clickCount = 0,
    );
    if (_clickCount >= clickCountToSelectTime) {
      selectDate();
    }
  }

  Future<List<int>?> selectDate() {
    return Picker(
      adapter: DateTimePickerAdapter(
        type: PickerDateTimeType.kYMDHM,
        value: _setDateTime,
      ),
      title: const Text("Select Time"),
      onConfirm: (Picker picker, List<int> selected) => setState(
        () {
          _setDateTime = DateTime(1900 + selected[0], 1 + selected[1],
              1 + selected[2], selected[3], selected[4]);
          _isChangeDateTime = true;
          _isSelectDateTime = true;
        },
      ),
    ).showDialog(context);
  }
}
